#include <stdint.h>
#include <board.h>
#include <delay.h>
#include <serial.h>
#include <lcd.h>
#include <stdbool.h>
#include <stdio.h>
#include "at45db321d.h"

/*****************************************************************************
 ** Function name:		flashmem_init
 **
 ** Descriptions:		Enable Flash Chipselect and Reset pins
 **
 ** parameters:			None
 **
 ** Returned value:		None
 **
 *****************************************************************************/
void flashmem_init(void)
{
	/* Initialize GPIO controlle */
	gpio_init();
	
	/* Enable Chipselect and FlashReset pins */
	gpio_enable_pin(FCS);
	gpio_enable_pin(FRST);
	
	/* Set Ouput Chipselect and FlashReset pins */
	gpio_direction_output(FCS, 1);
	gpio_direction_output(FRST, 1);
	
	/* set FlashReset pin into high */
	gpio_set_pin(FRST, 1); // equal to - gpio_direction_output(FRST, 1)
}

/*****************************************************************************
 ** Function name:		PageRead
 **
 ** Descriptions:		Read a page from flash, the first
 **				parameter is the Page Address, the 2nd parameter is
 **				starting byte address within the page. the third 
 **                             parameter is the buffer pointer to copy the page.
 **
 ** parameters:			Page Address, byte address, Buffer pointer.
 **
 ** Returned value:		None.
 ** 
 *****************************************************************************/
void PageRead(uint32_t PageAdr, uint16_t ByteAdr, uint8_t * buf)
{
	uint8_t rdateseq[8];
	
	/* Send page read command, page address, byte address 
	   one by one MSB First */
	rdateseq[0] = DF_PAGE_READ;	// D2H : 528 byte per page
	rdateseq[1] = (uint8_t)(PageAdr >> 6);
	rdateseq[2] = (uint8_t)(PageAdr << 2 | ByteAdr >> 8);
	rdateseq[3] = (uint8_t) ByteAdr;
	
	/* Four don't care bytes that follow the address bytes are needed to 
	   initialize the read operation. */
	rdateseq[4] = 0x00;
	rdateseq[5] = 0x00;
	rdateseq[6] = 0x00;
	rdateseq[7] = 0x00;
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	spi_write(rdateseq, 8);
	spi_read(buf, PGSIZE);
	
	/* Set chip select hogh */
	gpio_set_pin(FCS, 1);	
} 

/*****************************************************************************
 ** Function name:		PageWrite
 **
 ** Descriptions:		Write block of datas to a particular page, the first
 **				parameter is the Page Address, the 2nd parameter is
 **				starting byte address within the page. the third 
 **                             parameter is the buffer pointer will be write to page.
 **
 ** parameters:			Page Address, byte address, Buffer pointer.
 ** Returned value:		None
 ** 
 *****************************************************************************/ 
void PageWrite(uint32_t PageAdr, uint16_t ByteAdr, uint8_t * buf) 
{
	uint8_t wdateseq[4];
	
	/* Send page write command, page address, byte address 
	   one by one MSB First */ 
	wdateseq[0] = DF_PAGE_WRITE;  // 82H : 528 byte per page
	wdateseq[1] = (uint8_t)(PageAdr >> 6);
	wdateseq[2] = (uint8_t)(PageAdr << 2 | ByteAdr >> 8);
	wdateseq[3] = (uint8_t) ByteAdr;
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	spi_write(wdateseq, 4);
	spi_write(buf, PGSIZE);	
	/* Set chip select high */
	gpio_set_pin(FCS, 1);	

	/*monitor the status register,
	  wait until busy-flag is high */
	while(!ReadStatus());	
}


/*****************************************************************************
 ** Function name:		BufferRead
 **
 ** Descriptions:		Read block of data from Buffer, the first
 **			        parameter is Buffer number 0 or 1, the 2nd parameter is
 **				sarting byte address within the buffer. the third 
 **                             parameter is the buffer pointer to copy the Buffer.
 **
 ** parameters:			Buffer number, byte address, Buffer pointer.
 ** Returned value:		None.
 ** 
 *****************************************************************************/ 

void BufferRead(uint8_t Bufno, uint16_t ByteAdr, uint8_t * buf) 
{
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);
	
	if(Bufno == 1)
		spi_tx_rx(DF_BUFFER_1_READ);
	
	else
		spi_tx_rx(DF_BUFFER_2_READ);
	
	spi_tx_rx(0x00);
	
	/*upper part of internal buffer address */
	spi_tx_rx((ByteAdr & 0x0300) >> 8);	
	
	/*lower part of internal buffer address */
	spi_tx_rx((uint8_t)(ByteAdr));	
	
	/*don't cares */
	spi_tx_rx(0x00);		
	spi_read(buf, PGSIZE);
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
}


/*****************************************************************************
 ** Function name:		BufferWrite
 **
 ** Descriptions:		Write block of data to buffer, the first
 **				parameter is Buffer number 0 or 1, the 2nd parameter is
 **				starting byte address within buffer. the third 
 **                             parameter is the buffer pointer to copy to the Buffer.
 **
 ** parameters:			Buffer number, byte address, Buffer pointer.
 **
 ** Returned value:		None
 ** 
 *****************************************************************************/ 
void BufferWrite(uint8_t Bufno, uint16_t ByteAdr, uint8_t * buf) 
{
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);
	
	if(Bufno == 1)
		spi_tx_rx(DF_BUFFER_1_WRITE);
	
	else
		spi_tx_rx(DF_BUFFER_2_WRITE);
  
	spi_tx_rx(0x00);
	
	/* upper part of internal buffer address */
	spi_tx_rx((ByteAdr & 0x0300) >> 8);	
	
	/* lower part of internal buffer address */
	spi_tx_rx((uint8_t)(ByteAdr));	
	
	/* don't care bytes */
	spi_tx_rx(0x00);
	
	/* read data byte */
	spi_write(buf, PGSIZE);	
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
}

/*****************************************************************************
 ** Function name:		BufferToPage
 **
 ** Descriptions:		Write block of data from buffer to Flash page, the first
 **				parameter is Buffer number 0 or 1, the 2nd parameter is
 **                             page Address of flash.
 **
 ** parameters:			Buffer number, page Address
 **
 ** Returned value:		None
 ** 
 *****************************************************************************/ 
void BufferToPage(uint8_t BufferNum, uint32_t PageAdr) 
{
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	
	if(BufferNum == 1)
		spi_tx_rx(DF_BUFFER_1_TO_PAGE_WITH_ERASE);
	
	else
		spi_tx_rx(DF_BUFFER_2_TO_PAGE_WITH_ERASE);
	
	spi_tx_rx((PageAdr & 0x1FC0) >> 6);
	spi_tx_rx((PageAdr & 0x003F) << 2);
	
	/* don´t care bytes */ 
	spi_tx_rx(0x00);
	spi_tx_rx(0x00);
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
	
	/* monitor the status register, wait until busy-flag is high */
	while(!ReadStatus());	
}


/*****************************************************************************
 ** Function name:		PageToBuffer
 **
 ** Descriptions:		Read one page from Flash to specified buffer. The first
 **                             parameter is Buffer number 0 or 1, the 2nd parameter is
 **                             page Address of flash.
 **
 ** parameters:			Buffer number, page Address
 **
 ** Returned value:		None
 ** 
 *****************************************************************************/ 
void PageToBuffer(uint8_t BufferNum, uint32_t PageAdr) 
{

	/* Set chip select low */
	gpio_set_pin(FCS, 0);	

	if(BufferNum == 1)
		spi_tx_rx(DF_TRANSFER_PAGE_TO_BUFFER_1);
	else
		spi_tx_rx(DF_TRANSFER_PAGE_TO_BUFFER_2);
	
	spi_tx_rx((PageAdr & 0x1FC0) >> 6);
	spi_tx_rx((PageAdr & 0x003F) << 2);
	
	/* don´t care byte */
	spi_tx_rx(0x00);	
	
	/*release SPI bus for use by other sensors  */ 
	gpio_set_pin(FCS, 1);
	
	/* monitor the status register, wait until busy-flag is high */ 
	while(!ReadStatus());
	
}


/*****************************************************************************
 ** Function name:		PageErase
 **
 ** Descriptions:		Erase A particular page 
 **
 ** parameters:			page Address
 **
 ** Returned value:		None
 ** 
 *****************************************************************************/ 
void PageErase(uint32_t PageAdr) 
{
	
	gpio_set_pin(FCS, 0);
	spi_tx_rx(DF_PAGE_ERASE);
	spi_tx_rx((PageAdr & 0x1FC0) >> 6);
	spi_tx_rx((PageAdr & 0x003F) << 2);
	
	/* don´t care bytes */
	spi_tx_rx(0x00);		
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
	
	/* monitor the status register, wait until busy-flag is high */ 
	while(!ReadStatus());
}


/*****************************************************************************
 ** Function name:		ChipErase
 **
 ** Descriptions:		Erase Complete Flash 
 **
 ** parameters:			None 
 **
 ** Returned value:		None
 **
 *****************************************************************************/ 
void ChipErase(void) 
{
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	
	/* opcodes for chip erase */ 
	spi_tx_rx(DF_CHIP_ERASE_0);
	spi_tx_rx(DF_CHIP_ERASE_1);
	spi_tx_rx(DF_CHIP_ERASE_2);
	spi_tx_rx(DF_CHIP_ERASE_3);
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
  
	/* monitor the status register, wait until busy-flag is high */ 
	while(!ReadStatus());
}


/*****************************************************************************
 ** Function name:		ReadManufacturerID
 **
 ** Descriptions:		Reading Manufacturer and Device ID from Flash to 
 **                             the buffer,the parameter is buffer to coy the datas.
 **
 ** parameters:			Buffer pointer 
 **
 ** Returned value:		None.
 ** 
 *****************************************************************************/ 
void ReadManufacturerID(uint8_t * buf) 
{
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	spi_tx_rx(DF_READ_MANUFACTURER_AND_DEVICE_ID);
	buf[0] = spi_tx_rx(0xff);
	buf[1] = spi_tx_rx(0xff);
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
} 

/*****************************************************************************
 ** Function name:		ReadStatusReg
 **
 ** Descriptions:		Reading Status register content
 **
 ** parameters:			None
 **
 ** Returned value:		Status register content.
 ** 
 *****************************************************************************/ 
uint8_t ReadStatusReg(void) 
{
	uint8_t tmp;
	
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	
	/* Read status command */ 
	spi_tx_rx(DF_STATUS_REGISTER_READ);
	tmp = spi_tx_rx(0x00);	
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
	return tmp;
}


/*****************************************************************************
 ** Function name:		ReadStatus
 **
 ** Descriptions:		Return 0 if Flash Busy else 1
 **
 ** parameters:			None
 **
 ** Returned value:		0 or 1
 ** 
 *****************************************************************************/ 
uint8_t ReadStatus(void) 
{
	
	/* We only want to extract the READY/BUSY bit */
	return(ReadStatusReg() & 0x80);	
}


/*****************************************************************************
 ** Function name:		WaitReady
 **
 ** Descriptions:		Wait till the Flash is Busy
 **
 ** parameters:			None
 **
 ** Returned value:		None
 **
 *****************************************************************************/ 
void WaitReady(void) 
{
	
	/* Wait till the Flash is Busy */ 
	while(!ReadStatus());
}


/*****************************************************************************
 ** Function name:		powerdown
 **
 ** Descriptions:		powerdown the Flash 
 **
 ** parameters:			None
 **
 ** Returned value:		None
 ** 
 *****************************************************************************/ 
void powerdown(void) 
{
	/* Set chip select low */
	gpio_set_pin(FCS, 0);
	
	/* Send power down command */
	spi_tx_rx(POWERDOWN);	

	/* Set chip select high */
	gpio_set_pin(FCS, 1);	
} 

/*****************************************************************************
 ** Function name:		powerresume
 **
 ** Descriptions:		powerresume the Flash 
 **
 ** parameters:			None
 **
 ** Returned value:		None
 ** 
 *****************************************************************************/
void powerresume(void) 
{
	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	
        /*Send power resume command */
	spi_tx_rx(POWERRESUME);	

        /* Set chip select high */
	gpio_set_pin(FCS, 1);	
} 

/*****************************************************************************
 ** Function name:		ByteRead
 **
 ** Descriptions:		Read a Byte from flash, the first
 **				parameter is the Page Address, the 2nd parameter is
 **				byte address within the page. 
 **
 ** parameters:			Page Address, byte address
 **
 ** Returned value:		byte data
 **
*****************************************************************************/ 
uint8_t ByteRead(uint32_t PageAdr, uint16_t ByteAdr) 
{
	uint8_t rdateseq[8];
	uint8_t temp;
	
	/* Send page read command, page address, byte address 
	   one by one MSB First */ 
	rdateseq[0] = DF_PAGE_READ;
	rdateseq[1] = (uint8_t)(PageAdr >> 6);
	rdateseq[2] = (uint8_t)(PageAdr << 2 | ByteAdr >> 8);
	rdateseq[3] = (uint8_t) ByteAdr;
	
	/*Four dont care bytes that follow the address bytes are needed to 
	  initialize the read operation. */ 
	rdateseq[4] = 0x00;
	rdateseq[5] = 0x00;
	rdateseq[6] = 0x00;
	rdateseq[7] = 0x00;

        /* Set chip select low */
	gpio_set_pin(FCS, 0);	

        /*write page and byte address to read a data byte */
	spi_write(rdateseq, 8);	

        /* reading data from the given address */
	temp = spi_tx_rx(0xFF);	
	
	/* Set chip select high */
	gpio_set_pin(FCS, 1);	
	return temp;
}

